@@ -5,13 +5,18 @@
 #
 # ----------
 
+from .err import Abort
+
 class FlowContext:
     def __init__(self, state: dict=None):
         super().__init__()
-        self._state = {}
-        if state is not None:
-            self._state.update(state)
+        self._state = dict(state or ()) # make a clone
 
     @property
-    def state(self):
+    def state(self) -> dict:
+        'get the state dict.'
         return self._state
+
+    def abort(self, reason):
+        'abort the flow by raise a `Abort`.'
+        raise Abort(reason=reason)
